CREATE TABLE items (
	item        SERIAL PRIMARY KEY,
	title       TEXT   NOT NULL,
	category    TEXT   NOT NULL,
	description TEXT,

	begin_hour  INT    NOT NULL DEFAULT 5,
	end_hour    INT    NOT NULL DEFAULT 23,
	min_hours   INT    NOT NULL DEFAULT 1,
	max_hours   INT    NOT NULL DEFAULT 24
);

CREATE TABLE bookings (
	item       INT     NOT NULL REFERENCES items,
	name       TEXT    NOT NULL,
	begin_time BIGINT  NOT NULL,
	end_time   BIGINT  NOT NULL
);
